@@ -91,8 +91,8 @@
  * do the following:
  *
  *   - Use a global cache to share font info structs across different widgets.
- *     We use cairo font options, resolution, and font description as the key
- *     for our hash table.
+ *     We use pango language, cairo font options, resolution, and font description
+ *     as the key for our hash table.
  *
  *   - When a font info struct is no longer used by any widget, we delay
  *     destroying it for a while (FONT_CACHE_TIMEOUT seconds).  This is
@@ -270,6 +270,8 @@ font_info_cache_ascii (struct font_info *info)
 	cairo_scaled_font_t *scaled_font;
 	const char *text;
 	gboolean more;
+	PangoLanguage *language;
+	gboolean latin_uses_default_language;
 	
 	/* We have info->layout holding most ASCII characters.  We want to
 	 * cache as much info as we can about the ASCII letters so we don't
@@ -279,6 +281,11 @@ font_info_cache_ascii (struct font_info *info)
 	if (pango_layout_get_unknown_glyphs_count (info->layout) != 0)
 		return;
 
+	language = pango_context_get_language (pango_layout_get_context (info->layout));
+	if (language == NULL)
+		language = pango_language_get_default ();
+	latin_uses_default_language = pango_language_includes_script (language, PANGO_SCRIPT_LATIN);
+
 	text = pango_layout_get_text (info->layout);
 
 	line = pango_layout_get_line_readonly (info->layout, 0);
@@ -316,9 +323,11 @@ font_info_cache_ascii (struct font_info *info)
 		glyph = glyph_string->glyphs[iter.start_glyph].glyph;
 		geometry = &glyph_string->glyphs[iter.start_glyph].geometry;
 
-		/* Only cache non-common characters as common characters get
-		 * their font from their neighbors */
-		if (pango_script_for_unichar (c) <= PANGO_SCRIPT_INHERITED)
+		/* If not using the default locale language, only cache non-common
+		 * characters as common characters get their font from their neighbors
+		 * and we don't want to force Latin on them. */
+		if (!latin_uses_default_language &&
+		    pango_script_for_unichar (c) <= PANGO_SCRIPT_INHERITED)
 			continue;
 
 		/* Only cache simple glyphs */
@@ -356,15 +365,13 @@ static void
 font_info_measure_font (struct font_info *info)
 {
 	PangoRectangle logical;
-	gunichar full_codepoints[] = {VTE_DRAW_DOUBLE_WIDE_IDEOGRAPHS};
-	GString *full_string;
-	gint single_width, full_width;
-	guint i;
 
 	/* Estimate for ASCII characters. */
 	pango_layout_set_text (info->layout, VTE_DRAW_SINGLE_WIDE_CHARACTERS, -1);
 	pango_layout_get_extents (info->layout, NULL, &logical);
-	single_width = howmany (logical.width, strlen(VTE_DRAW_SINGLE_WIDE_CHARACTERS));
+	/* We don't do CEIL for width since we are averaging;
+	 * rounding is more accurate */
+	info->width  = PANGO_PIXELS (howmany (logical.width, strlen(VTE_DRAW_SINGLE_WIDE_CHARACTERS)));
 	info->height = PANGO_PIXELS_CEIL (logical.height);
 	info->ascent = PANGO_PIXELS_CEIL (pango_layout_get_baseline (info->layout));
 
@@ -372,25 +379,6 @@ font_info_measure_font (struct font_info *info)
 	 * info for them */
 	font_info_cache_ascii (info);
 
-	/* Estimate for CJK characters. */
-	full_width = single_width * 2;
-	full_string = g_string_new(NULL);
-	for (i = 0; i < G_N_ELEMENTS (full_codepoints); i++) {
-		g_string_append_unichar (full_string, full_codepoints[i]);
-	}
-	pango_layout_set_text (info->layout, full_string->str, -1);
-	g_string_free (full_string, TRUE);
-	pango_layout_get_extents (info->layout, NULL, &logical);
-	full_width = howmany (logical.width, G_N_ELEMENTS(full_codepoints));
-
-	/* If they're the same, then we have a screwy font. */
-	if (full_width == single_width) {
-		single_width = (full_width + 1) / 2;
-	}
-
-	/* We don't do CEIL for width since we are averaging; rounding is
-	 * more accurate */
-	info->width = PANGO_PIXELS (single_width);
 
 	if (info->height == 0) {
 		info->height = PANGO_PIXELS_CEIL (logical.height);
@@ -564,7 +552,8 @@ context_hash (PangoContext *context)
 {
 	return pango_units_from_double (pango_cairo_context_get_resolution (context))
 	     ^ pango_font_description_hash (pango_context_get_font_description (context))
-	     ^ cairo_font_options_hash (pango_cairo_context_get_font_options (context));
+	     ^ cairo_font_options_hash (pango_cairo_context_get_font_options (context))
+	     ^ GPOINTER_TO_UINT (pango_context_get_language (context));
 }
 
 static gboolean
@@ -573,7 +562,8 @@ context_equal (PangoContext *a,
 {
 	return pango_cairo_context_get_resolution (a) == pango_cairo_context_get_resolution (b)
 	    && pango_font_description_equal (pango_context_get_font_description (a), pango_context_get_font_description (b))
-	    && cairo_font_options_equal (pango_cairo_context_get_font_options (a), pango_cairo_context_get_font_options (b));
+	    && cairo_font_options_equal (pango_cairo_context_get_font_options (a), pango_cairo_context_get_font_options (b))
+	    && pango_context_get_language (a) == pango_context_get_language (b);
 }
 
 static struct font_info *
@@ -605,7 +595,8 @@ font_info_find_for_context (PangoContext *context)
 static struct font_info *
 font_info_create_for_context (PangoContext               *context,
 			      const PangoFontDescription *desc,
-			      VteTerminalAntiAlias        antialias)
+			      VteTerminalAntiAlias        antialias,
+			      PangoLanguage              *language)
 {
 	if (!PANGO_IS_CAIRO_FONT_MAP (pango_context_get_font_map (context))) {
 		/* Ouch, Gtk+ switched over to some drawing system?
@@ -616,9 +607,12 @@ font_info_create_for_context (PangoContext               *context,
 	}
 
 	pango_context_set_base_dir (context, PANGO_DIRECTION_LTR);
+
 	if (desc)
 		pango_context_set_font_description (context, desc);
 
+	pango_context_set_language (context, language);
+
 	switch (antialias) {
 		cairo_font_options_t *font_options;
 		cairo_antialias_t cr_aa;
@@ -657,10 +651,11 @@ font_info_create_for_context (PangoContext               *context,
 static struct font_info *
 font_info_create_for_screen (GdkScreen                  *screen,
 			     const PangoFontDescription *desc,
-			     VteTerminalAntiAlias        antialias)
+			     VteTerminalAntiAlias        antialias,
+			     PangoLanguage              *language)
 {
 	return font_info_create_for_context (gdk_pango_context_get_for_screen (screen),
-					     desc, antialias);
+					     desc, antialias, language);
 }
 
 static struct font_info *
@@ -669,8 +664,9 @@ font_info_create_for_widget (GtkWidget                  *widget,
 			     VteTerminalAntiAlias        antialias)
 {
 	GdkScreen *screen = gtk_widget_get_screen (widget);
+	PangoLanguage *language = pango_context_get_language (gtk_widget_get_pango_context (widget));
 
-	return font_info_create_for_screen (screen, desc, antialias);
+	return font_info_create_for_screen (screen, desc, antialias, language);
 }
 
 static struct unichar_info *
